import { Component, Element, Event, h, Host, Listen, Method, Prop, Watch } from '@stencil/core';
import { Color } from '../../utils/utils';
export class CollapsibleDashboard {
  constructor() {
    this.direction = 'ltr';
    this.collapse = false;
    this.open = false;
    this.sidePanelBgColorObject = new Color('#e5e5e5');
    this.sidePanelColorObject = new Color('#232323');
    this.mobileNavBgColorObject = new Color('#232323');
    this.mobileNavColorObject = new Color('#e5e5e5');
  }
  collapseChangeHandler(collapse) {
    this.collapseChange.emit(collapse);
  }
  setCollapse(newVal) {
    this.collapseChangeHandler(newVal);
  }
  componentWillLoad() {
    this.direction = window.getComputedStyle(this.element).direction;
    this.setSidePanelBgColor(this.sidePanelBgColor);
    this.setMobileNavBgColor(this.mobileNavBgColor);
  }
  componentDidRender() {
    var _a, _b;
    this.dashboardWrapper = ((_a = this.element.shadowRoot) !== null && _a !== void 0 ? _a : this.element).querySelector('.dashboard-wrapper');
    this.sidePanel = ((_b = this.element.shadowRoot) !== null && _b !== void 0 ? _b : this.element).querySelector('.side-panel');
    // this.mobileNav = (this.element.shadowRoot ?? this.element).querySelector('.mobile-nav');
    this.updateCssVariables();
    this.windowResize();
  }
  setSidePanelBgColor(newVal) {
    if (newVal) {
      this.sidePanelBgColorObject.parse(newVal);
    }
    this.sidePanelBgColor = this.sidePanelBgColorObject.toRGBString();
    this.updateCssVariables();
  }
  setSidePanelColor(newVal) {
    if (newVal) {
      this.sidePanelColorObject.parse(newVal);
      this.sidePanelColor = this.sidePanelColorObject.toRGBString();
    }
    this.updateCssVariables();
  }
  setMobileNavBgColor(newVal) {
    if (newVal) {
      this.mobileNavBgColorObject.parse(newVal);
    }
    this.mobileNavBgColor = this.mobileNavBgColorObject.toRGBString();
    this.updateCssVariables();
  }
  setMobileNavColor(newVal) {
    if (newVal) {
      this.mobileNavColorObject.parse(newVal);
      this.mobileNavColor = this.mobileNavColorObject.toRGBString();
    }
    this.updateCssVariables();
  }
  updateCssVariables() {
    if (this.dashboardWrapper) {
      const mobileNavColor = new Color(this.getMobileNavColor());
      const mobileNavBg = new Color(this.getMobileNavBgColor());
      const sidePanelColor = new Color(this.getSidePanelColor());
      const sidePanelBg = new Color(this.getSidePanelBgColor());
      const mobileNavColorRGBA = mobileNavColor.getRGBA();
      const mobileNavBgRGBA = mobileNavBg.getRGBA();
      const sidePanelColorRGBA = sidePanelColor.getRGBA();
      const sidePanelBgRGBA = sidePanelBg.getRGBA();
      // const diff: {r?: number; g?: number, b?: number} = {};
      for (const key of ['r', 'g', 'b', 'a']) {
        this.dashboardWrapper.style.setProperty(`--mobile-nav-color-${key}`, mobileNavColorRGBA[key]);
        this.dashboardWrapper.style.setProperty(`--mobile-nav-bg-${key}`, mobileNavBgRGBA[key]);
        this.dashboardWrapper.style.setProperty(`--side-panel-color-${key}`, sidePanelColorRGBA[key]);
        this.dashboardWrapper.style.setProperty(`--side-panel-bg-${key}`, sidePanelBgRGBA[key]);
        this.dashboardWrapper.style.setProperty(`--color-diff-${key}`, (sidePanelColorRGBA[key] - mobileNavColorRGBA[key]).toFixed(0));
      }
    }
  }
  async getSidePanelBgColorPromise() {
    return this.getSidePanelBgColor();
  }
  getSidePanelBgColor() {
    return this.sidePanelBgColor;
  }
  getDefaultSidePanelColor() {
    return this.sidePanelBgColorObject.isDark() ? '#efefef' : '#232323';
  }
  async getSidePanelColorPromise() {
    return this.getSidePanelColor();
  }
  getSidePanelColor() {
    var _a;
    return (_a = this.sidePanelColor) !== null && _a !== void 0 ? _a : this.getDefaultSidePanelColor();
  }
  async getMobileNavBgColorPromise() {
    return this.getMobileNavBgColor();
  }
  getMobileNavBgColor() {
    return this.mobileNavBgColor;
  }
  getDefaultMobileNavColor() {
    return this.mobileNavBgColorObject.isDark() ? '#efefef' : '#232323';
  }
  async getMobileNavColorPromise() {
    return this.getMobileNavColor();
  }
  getMobileNavColor() {
    var _a;
    return (_a = this.mobileNavColor) !== null && _a !== void 0 ? _a : this.getDefaultMobileNavColor();
  }
  touchstart(event) {
    var _a, _b;
    this.touchX = (_a = event.touches[0]) === null || _a === void 0 ? void 0 : _a.clientX;
    this.touchY = (_b = event.touches[0]) === null || _b === void 0 ? void 0 : _b.clientY;
    this.width = this.sidePanel.clientWidth;
    this.factor = (this.direction === 'ltr' ? 1 : -1) / this.width;
    if (this.open) {
      return;
    }
    if ((this.touchX >= 20 && this.direction === 'ltr') || (this.touchX <= this.dashboardWrapper.clientWidth - 20 && this.direction === 'rtl')) {
      this.touchX = Infinity;
    }
  }
  touchmove(event) {
    var _a, _b;
    if (!isFinite(this.touchX)) {
      return;
    }
    this.deltaY = this.touchY - ((_a = event.touches[0]) === null || _a === void 0 ? void 0 : _a.clientY);
    this.deltaX = Math.max(0, Math.min(1, (this.touchX - ((_b = event.touches[0]) === null || _b === void 0 ? void 0 : _b.clientX)) * this.factor + +!this.open));
    if (Math.abs(this.deltaY) > 50) {
      this.dashboardWrapper.classList.remove('dragging');
      this.dashboardWrapper.style.removeProperty('--progression');
      this.touchX = Infinity;
      this.deltaX = Infinity;
    }
    else if (Math.abs(this.deltaX) > 0.05) {
      this.dashboardWrapper.classList.add('dragging');
      this.dashboardWrapper.style.setProperty('--progression', this.deltaX.toString());
      event.preventDefault();
    }
  }
  touchend() {
    this.touchX = Infinity;
    this.dashboardWrapper.classList.remove('dragging');
    this.dashboardWrapper.style.removeProperty('--progression');
    if (isFinite(this.deltaX)) {
      if (Math.abs(this.deltaX) > 0.5 && this.open) {
        this.open = false;
      }
      else if (Math.abs(this.deltaX) < 0.5 && !this.open) {
        this.open = true;
      }
    }
    this.deltaX = Infinity;
  }
  windowResize() {
    var _a;
    (_a = this.dashboardWrapper) === null || _a === void 0 ? void 0 : _a.style.setProperty('--vh', `${window.innerHeight * 0.01}px`);
  }
  watchDir(newVal) {
    this.direction = newVal;
  }
  chevronLeft() {
    return (h("svg", { "aria-hidden": "true", focusable: "false", "data-prefix": "fas", "data-icon": "chevron-left", role: "img", xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 320 512", class: "svg-inline--fa fa-chevron-left fa-w-10 fa-7x" },
      h("path", { fill: this.getSidePanelColor(), d: "M34.52 239.03L228.87 44.69c9.37-9.37 24.57-9.37 33.94 0l22.67 22.67c9.36 9.36 9.37 24.52.04 33.9L131.49 256l154.02 154.75c9.34 9.38 9.32 24.54-.04 33.9l-22.67 22.67c-9.37 9.37-24.57 9.37-33.94 0L34.52 272.97c-9.37-9.37-9.37-24.57 0-33.94z", class: "" })));
  }
  chevronRight() {
    return (h("svg", { "aria-hidden": "true", focusable: "false", "data-prefix": "fas", "data-icon": "chevron-right", role: "img", xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 320 512", class: "svg-inline--fa fa-chevron-right fa-w-10 fa-7x" },
      h("path", { fill: this.getSidePanelColor(), d: "M285.476 272.971L91.132 467.314c-9.373 9.373-24.569 9.373-33.941 0l-22.667-22.667c-9.357-9.357-9.375-24.522-.04-33.901L188.505 256 34.484 101.255c-9.335-9.379-9.317-24.544.04-33.901l22.667-22.667c9.373-9.373 24.569-9.373 33.941 0L285.475 239.03c9.373 9.372 9.373 24.568.001 33.941z", class: "" })));
  }
  menuIcon() {
    return (h("svg", { "aria-hidden": "true", focusable: "false", "data-prefix": "fas", "data-icon": "minus", role: "img", xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 512 512", class: `svg-inline--fa fa-menu fa-w-14 fa-7x` },
      h("g", null,
        h("path", { fill: this.open ? this.getSidePanelColor() : this.getSidePanelBgColor(), d: "M448,64H64c-17.7,0-32,14.3-32,32v32c0,17.7,14.3,32,32,32h384c17.7,0,32-14.3,32-32V96C480,78.3,465.7,64,448,64z" })),
      h("g", null,
        h("path", { fill: this.open ? this.getSidePanelColor() : this.getSidePanelBgColor(), d: "M447.5,208.1h-384c-17.7,0-32,14.3-32,32v32c0,17.7,14.3,32,32,32h384c17.7,0,32-14.3,32-32v-32 C479.5,222.4,465.2,208.1,447.5,208.1z" })),
      h("g", null,
        h("path", { fill: this.open ? this.getSidePanelColor() : this.getSidePanelBgColor(), d: "M448,350H64c-17.7,0-32,14.3-32,32v32c0,17.7,14.3,32,32,32h384c17.7,0,32-14.3,32-32v-32C480,364.3,465.7,350,448,350z" }))));
  }
  render() {
    return (h(Host, null,
      h("div", { class: `dashboard-wrapper ${this.collapse ? ' collapse-panel ' : ''}${this.open ? ' open ' : ''}`, dir: this.direction === 'ltr' ? 'ltr' : 'rtl' },
        h("div", { class: "dashboard-container" },
          h("nav", { class: `side-panel`, style: { 'background-color': this.getSidePanelBgColor(), 'color': this.getSidePanelColor() } },
            h("div", { class: "side-panel-header" },
              h("div", { class: "side-panel-header-container" },
                h("slot", { name: "side-panel-header" })),
              h("button", { class: `menu-button ${this.open ? 'open' : ''}`, onClick: () => (this.open = !this.open) }, this.menuIcon())),
            h("div", { class: "side-panel-content" },
              h("slot", { name: "side-panel-content" })),
            h("div", { class: `collapse-panel-button ${this.sidePanelBgColorObject.isDark() ? 'dark' : 'light'}` },
              h("button", { onClick: () => (this.collapse = !this.collapse) }, this.direction === 'ltr' ? this.chevronLeft() : this.chevronRight()))),
          h("main", { class: "main" },
            h("nav", { class: "mobile-nav", style: { 'background-color': this.getMobileNavBgColor(), 'color': this.getMobileNavColor() } },
              h("div", { class: "mobile-nav-container" },
                h("slot", { name: "mobile-nav" }))),
            h("slot", null))))));
  }
  static get is() { return "collapsible-dashboard"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["collapsible-dashboard.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["collapsible-dashboard.css"]
  }; }
  static get properties() { return {
    "direction": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "'rtl' | 'ltr'",
        "resolved": "\"ltr\" | \"rtl\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "dir",
      "reflect": false,
      "defaultValue": "'ltr'"
    },
    "collapse": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "collapse",
      "reflect": true,
      "defaultValue": "false"
    },
    "open": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "sidePanelBgColor": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "side-panel-bg-color",
      "reflect": false
    },
    "sidePanelColor": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "side-panel-color",
      "reflect": false
    },
    "mobileNavBgColor": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "mobile-nav-bg-color",
      "reflect": false
    },
    "mobileNavColor": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "mobile-nav-color",
      "reflect": false
    }
  }; }
  static get events() { return [{
      "method": "collapseChange",
      "name": "collapseChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "getSidePanelBgColorPromise": {
      "complexType": {
        "signature": "() => Promise<string>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<string>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getSidePanelColorPromise": {
      "complexType": {
        "signature": "() => Promise<string>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<string>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getMobileNavBgColorPromise": {
      "complexType": {
        "signature": "() => Promise<string>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<string>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getMobileNavColorPromise": {
      "complexType": {
        "signature": "() => Promise<string>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<string>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "element"; }
  static get watchers() { return [{
      "propName": "collapse",
      "methodName": "setCollapse"
    }, {
      "propName": "sidePanelBgColor",
      "methodName": "setSidePanelBgColor"
    }, {
      "propName": "sidePanelColor",
      "methodName": "setSidePanelColor"
    }, {
      "propName": "mobileNavBgColor",
      "methodName": "setMobileNavBgColor"
    }, {
      "propName": "mobileNavColor",
      "methodName": "setMobileNavColor"
    }, {
      "propName": "direction",
      "methodName": "watchDir"
    }]; }
  static get listeners() { return [{
      "name": "touchstart",
      "method": "touchstart",
      "target": "window",
      "capture": true,
      "passive": false
    }, {
      "name": "touchmove",
      "method": "touchmove",
      "target": "window",
      "capture": true,
      "passive": false
    }, {
      "name": "touchend",
      "method": "touchend",
      "target": "window",
      "capture": true,
      "passive": false
    }, {
      "name": "resize",
      "method": "windowResize",
      "target": "window",
      "capture": false,
      "passive": true
    }]; }
}
